/*-=-=-=-=-=-=-=-=-=-=-=- RCCL/RCI Notice of Copyright -=-=-=-=-=-=-=-=-=-=-*
 *									    *
 *	Copyright (C) 1991 by John E. Lloyd.				    *
 *									    *
 *	Information on copyright permissions, as well as a complete	    *
 *	disclaimer of warranty, is given in the file COPYRIGHT.		    *
 *									    *
 *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
#ifndef _TIMEVAL_H_
#define _TIMEVAL_H_

#include <sys/time.h>

struct timeval addTimeval();
struct timeval diffTimeval();
struct timeval negateTimeval();
struct timeval doubleToTimeval();
struct timeval rciWallClock();
extern double timevalToDouble();

#endif
